@@ -1,4 +1,6 @@
+import json
 import logging
+from urllib.parse import urlencode
 
 from django.db.models import BooleanField
 from django.urls import reverse
@@ -67,12 +69,24 @@ class AdminMixin:
 
     def changelist_view(self, request, extra_context=None):
         extra_context = extra_context or {}
-        url = reverse(
-            "data_browser:query_html",
-            args=[f"{self.model._meta.app_label}.{self.model.__name__}", ""],
-        )
-        args = getattr(self, "ddb_default_filters", "")
-        extra_context["ddb_url"] = f"{url}?{args}"
+
+        if not getattr(self, "ddb_ignore", False):
+            url = reverse(
+                "data_browser:query_html",
+                args=[f"{self.model._meta.app_label}.{self.model.__name__}", ""],
+            )
+            args = getattr(self, "ddb_default_filters", [])
+            params = urlencode(
+                [
+                    (
+                        f"{field}__{lookup}",
+                        value if isinstance(value, str) else json.dumps(value),
+                    )
+                    for field, lookup, value in args
+                ]
+            )
+            extra_context["ddb_url"] = f"{url}?{params}"
+
         return super().changelist_view(request, extra_context)
 
 
@@ -96,6 +110,9 @@ class AnnotationDescriptor:
     def __call__(self, obj):
         return getattr(obj, self.name)
 
+    def __getattr__(self, name):
+        return getattr(self.get_queryset, name)
+
 
 def ddb_hide(func):  # pragma: no cover
     logging.getLogger(__name__).warning(
