@@ -1,7 +1,6 @@
 from collections import defaultdict
 
 from django.contrib.admin import site
-from django.contrib.admin.options import InlineModelAdmin
 from django.contrib.admin.utils import flatten_fieldsets
 from django.contrib.auth.admin import UserAdmin
 from django.db import models
@@ -31,9 +30,11 @@ from .types import (
     DurationType,
     JSONType,
     NumberArrayType,
+    NumberChoiceArrayType,
     NumberChoiceType,
     NumberType,
     StringArrayType,
+    StringChoiceArrayType,
     StringChoiceType,
     StringType,
     UnknownType,
@@ -107,7 +108,7 @@ def _get_all_admin_fields(request):
         fields = admin.get_fieldsets(request, obj)
         for f in flatten_fieldsets(fields):
             # skip calculated fields on inlines
-            if not isinstance(admin, InlineModelAdmin) or hasattr(admin.model, f):
+            if all_ or hasattr(admin.model, f):
                 yield f
 
     def visible(model_admin, request):
@@ -130,9 +131,9 @@ def _get_all_admin_fields(request):
     hidden_fields = defaultdict(set)
     model_admins = {}
     for model, model_admin in site._registry.items():
-        model_admins[model] = model_admin
         if visible(model_admin, request):
-            all_admin_fields[model].update(from_fieldsets(model_admin, True))
+            model_admins[model] = model_admin
+            all_admin_fields[model].update(from_fieldsets(model_admin, False))
             all_admin_fields[model].update(model_admin.get_list_display(request))
             all_admin_fields[model].update(getattr(model_admin, "ddb_extra_fields", []))
             all_admin_fields[model].add(open_in_admin)
@@ -146,6 +147,7 @@ def _get_all_admin_fields(request):
                     except Exception as e:
                         debug_log(e)  # ignore things like GenericInlineModelAdmin
                     else:
+
                         if inline.model not in model_admins:  # pragma: no branch
                             model_admins[inline.model] = inline
                         all_admin_fields[inline.model].update(
@@ -162,6 +164,7 @@ def _get_all_admin_fields(request):
     for model, model_admin in model_admins.items():
         if isinstance(model_admin, AdminMixin):
             all_admin_fields[model].update(model_admin._ddb_annotations())
+        all_admin_fields[model].update(from_fieldsets(model_admin, True))
 
     # we always have id and never pk
     for fields in all_admin_fields.values():
@@ -177,15 +180,27 @@ def _get_all_admin_fields(request):
     return model_admins, all_admin_fields
 
 
+def _upper(name):
+    return name[0].upper() + name[1:]
+
+
 def _get_calculated_field(request, field_name, model_name, model, admin, model_fields):
     if isinstance(field_name, str):
         field_func = getattr(admin, field_name, None)
     else:
+        # an actual function in list_display
         field_func = field_name
-        field_name = getattr(
-            field_func, "short_description", slugify(str(field_func.__name__))
+        field_name = slugify(
+            getattr(field_func, "short_description", str(field_func.__name__))
         )
 
+    if getattr(field_func, "ddb_hide", False):
+        return None
+
+    pretty_name = _upper(
+        getattr(field_func, "short_description", field_name.replace("_", " "))
+    )
+
     if isinstance(field_func, AnnotationDescriptor):
         qs = admin_get_queryset(admin, request, [field_name])
 
@@ -205,16 +220,13 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
         return OrmAnnotatedField(
             model_name=model_name,
             name=field_name,
-            pretty_name=field_name,
+            pretty_name=pretty_name,
             type_=type_,
             field_type=field_type,
             admin=admin,
             choices=choices,
         )
     else:
-        if getattr(field_func, "ddb_hide", False):
-            return None
-
         if field_func is None:
 
             def field_func(obj):
@@ -224,7 +236,7 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
         return OrmCalculatedField(
             model_name=model_name,
             name=field_name,
-            pretty_name=field_name,
+            pretty_name=pretty_name,
             func=field_func,
         )
 
@@ -237,11 +249,28 @@ def _get_field_type(model, field_name, field):
     if isinstance(field, ArrayField) and isinstance(
         field.base_field, _STRING_FIELDS
     ):  # pragma: postgres
-        return StringArrayType, _fmt_choices(field.base_field.choices)
+        base_field, choices = _get_field_type(model, field_name, field.base_field)
+        array_types = {
+            StringType: StringArrayType,
+            NumberType: NumberArrayType,
+            StringChoiceType: StringChoiceArrayType,
+            NumberChoiceType: NumberChoiceArrayType,
+        }
+        if base_field in array_types:
+            return array_types[base_field], choices
+        else:
+            debug_log(
+                f"{model.__name__}.{field_name} unsupported subarray type {type(field.base_field).__name__}"
+            )
+            return UnknownType, None
+
     elif isinstance(field, ArrayField) and isinstance(
         field.base_field, _NUMBER_FIELDS
     ):  # pragma: postgres
-        return NumberArrayType, _fmt_choices(field.base_field.choices)
+        if field.base_field.choices:
+            return NumberChoiceArrayType, _fmt_choices(field.base_field.choices)
+        else:
+            return NumberArrayType, None
     elif isinstance(field, JSONField):
         res = JSONType
     elif field.__class__ in _FIELD_TYPE_MAP:
@@ -278,6 +307,7 @@ def _make_json_sub_module(model_name, field_types):
             pretty_name=field_name,
             type_=type_,
             rel_name=type_.name,
+            choices=None,
         )
 
     return OrmModel(fields)
@@ -292,29 +322,37 @@ def _get_fields_for_model(request, model, admin, admin_fields):
 
     for field_name in admin_fields[model]:
         field = model_fields.get(field_name)
+        pretty_name = (
+            _upper(getattr(field, "verbose_name", field_name)) if field else None
+        )
+        # FK's and OneToOne's
         if isinstance(field, (models.ForeignKey, OneToOneRel)):
             if field.related_model in admin_fields:
                 fields[field_name] = OrmFkField(
                     model_name=model_name,
                     name=field_name,
-                    pretty_name=field_name,
+                    pretty_name=pretty_name,
                     rel_name=get_model_name(field.related_model),
                 )
+        # ManyToMany
         elif isinstance(field, (ForeignObjectRel, models.ManyToManyField)):
-            pass  # TODO 2many support
+            pass
+        # Files and Images
         elif isinstance(field, models.FileField):
             fields[field_name] = OrmFileField(
                 model_name=model_name,
                 name=field_name,
-                pretty_name=field_name,
+                pretty_name=pretty_name,
                 django_field=field,
             )
+        # Calculated and annoted fields
         elif isinstance(field, type(None)):
             orm_field = _get_calculated_field(
                 request, field_name, model_name, model, admin, model_fields
             )
             if orm_field:
                 fields[orm_field.name] = orm_field
+        # Normal fields
         else:
             field_type, choices = _get_field_type(model, field_name, field)
 
@@ -330,7 +368,7 @@ def _get_fields_for_model(request, model, admin, admin_fields):
             fields[field_name] = OrmConcreteField(
                 model_name=model_name,
                 name=field_name,
-                pretty_name=field_name,
+                pretty_name=pretty_name,
                 type_=field_type,
                 rel_name=rel_name,
                 choices=choices,
