@@ -1,3 +1,4 @@
+import json
 from collections import defaultdict
 from dataclasses import dataclass
 from typing import Sequence, Tuple
@@ -125,12 +126,12 @@ def _get_django_aggregate(field_type, name):
 
 
 def _get_django_lookup(field_type, lookup, filter_value):
-    from .types import StringChoiceType, StringType
+    from .types import StringType
 
     if lookup == "field_equals":
         lookup, filter_value = filter_value
         return lookup, filter_value
-    elif field_type in [StringType, StringChoiceType]:
+    elif field_type == StringType:
         return (
             {
                 "equals": "iexact",
@@ -205,7 +206,13 @@ def get_fields_for_type(type_):
         func: OrmFunctionField(type_.name, func, _get_django_function(func)[1])
         for func in _TYPE_FUNCTIONS[type_]
     }
-    return {**aggregates, **functions}
+    others = {}
+    if type_.raw_type:
+        others["raw"] = OrmRawField(
+            type_.name, "raw", "raw", type_.raw_type, type_.raw_type.name, None
+        )
+
+    return {**aggregates, **functions, **others}
 
 
 @dataclass
@@ -214,7 +221,7 @@ class OrmBoundField:
     previous: "OrmBoundField"
     full_path: Sequence[str]
     pretty_path: Sequence[str]
-    queryset_path: str = None
+    queryset_path: Sequence[str]
     aggregate_clause: Tuple[str, models.Func] = None
     filter_: bool = False
     having: bool = False
@@ -224,11 +231,25 @@ class OrmBoundField:
     def path_str(self):
         return s(self.full_path)
 
+    @property
+    def queryset_path_str(self):
+        return s(self.queryset_path)
+
     @property
     def group_by(self):
         return self.field.can_pivot
 
+    def _lineage(self):
+        if self.previous:
+            return self.previous._lineage() + [self]
+        return [self]
+
     def annotate(self, request, qs):
+        for field in self._lineage():
+            qs = field._annotate(request, qs)
+        return qs
+
+    def _annotate(self, request, qs):
         return qs
 
     def __getattr__(self, name):
@@ -236,7 +257,9 @@ class OrmBoundField:
 
     @classmethod
     def blank(cls):
-        return cls(field=None, previous=None, full_path=[], pretty_path=[])
+        return cls(
+            field=None, previous=None, full_path=[], pretty_path=[], queryset_path=[]
+        )
 
     def get_format_hints(self, data):
         return self.type_.get_format_hints(self.path_str, data)
@@ -253,7 +276,12 @@ class OrmModel:
 
     @property
     def default_filters(self):
-        return getattr(self.admin, "ddb_default_filters", "")
+        ddb_default_filters = getattr(self.admin, "ddb_default_filters", [])
+        assert isinstance(ddb_default_filters, list)
+        return [
+            (f, l, v if isinstance(v, str) else json.dumps(v))
+            for (f, l, v) in ddb_default_filters
+        ]
 
 
 @dataclass
@@ -283,17 +311,17 @@ class OrmFkField(OrmBaseField):
 
     def bind(self, previous):
         previous = previous or OrmBoundField.blank()
-        full_path = previous.full_path + [self.name]
         return OrmBoundField(
             field=self,
             previous=previous,
-            full_path=full_path,
+            full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
+            queryset_path=previous.queryset_path + [self.name],
         )
 
 
 class OrmConcreteField(OrmBaseField):
-    def __init__(self, model_name, name, pretty_name, type_, rel_name, choices=None):
+    def __init__(self, model_name, name, pretty_name, type_, rel_name, choices):
         super().__init__(
             model_name,
             name,
@@ -307,13 +335,24 @@ class OrmConcreteField(OrmBaseField):
 
     def bind(self, previous):
         previous = previous or OrmBoundField.blank()
-        full_path = previous.full_path + [self.name]
         return OrmBoundField(
             field=self,
             previous=previous,
-            full_path=full_path,
+            full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=s(full_path),
+            queryset_path=previous.queryset_path + [self.name],
+            filter_=True,
+        )
+
+
+class OrmRawField(OrmConcreteField):
+    def bind(self, previous):
+        return OrmBoundField(
+            field=self,
+            previous=previous,
+            full_path=previous.full_path + [self.name],
+            pretty_path=previous.pretty_path + [self.pretty_name],
+            queryset_path=previous.queryset_path,
             filter_=True,
         )
 
@@ -330,13 +369,12 @@ class OrmCalculatedField(OrmBaseField):
 
     def bind(self, previous):
         previous = previous or OrmBoundField.blank()
-        full_path = previous.full_path + [self.name]
         return OrmBoundField(
             field=self,
             previous=previous,
-            full_path=full_path,
+            full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=s(previous.full_path + ["id"]),
+            queryset_path=previous.queryset_path + ["id"],
             model_name=self.model_name,
         )
 
@@ -358,14 +396,14 @@ class OrmCalculatedField(OrmBaseField):
 
 
 class OrmBoundAnnotatedField(OrmBoundField):
-    def annotate(self, request, qs):
+    def _annotate(self, request, qs):
         from .orm_results import admin_get_queryset
 
         return qs.annotate(
             **{
-                self.queryset_path: Subquery(
+                s(self.queryset_path): Subquery(
                     admin_get_queryset(self.admin, request, [self.name])
-                    .filter(pk=OuterRef(s(self.previous.full_path + ["id"])))
+                    .filter(pk=OuterRef(s(self.previous.queryset_path + ["id"])))
                     .values(self.name)[:1],
                     output_field=self.field_type,
                 )
@@ -375,13 +413,14 @@ class OrmBoundAnnotatedField(OrmBoundField):
 
 class OrmAnnotatedField(OrmBaseField):
     def __init__(
-        self, model_name, name, pretty_name, type_, field_type, admin, choices=None
+        self, model_name, name, pretty_name, type_, field_type, admin, choices
     ):
         super().__init__(
             model_name,
             name,
             pretty_name,
             type_=type_,
+            rel_name=type_.name,
             can_pivot=True,
             concrete=True,
             choices=choices or (),
@@ -398,7 +437,7 @@ class OrmAnnotatedField(OrmBaseField):
             previous=previous,
             full_path=full_path,
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=f"ddb_{s(full_path)}",
+            queryset_path=[s(["ddb"] + full_path)],
             filter_=True,
         )
 
@@ -406,7 +445,12 @@ class OrmAnnotatedField(OrmBaseField):
 class OrmFileField(OrmConcreteField):
     def __init__(self, model_name, name, pretty_name, django_field):
         super().__init__(
-            model_name, name, pretty_name, type_=HTMLType, rel_name=HTMLType.name
+            model_name,
+            name,
+            pretty_name,
+            type_=HTMLType,
+            rel_name=HTMLType.name,
+            choices=None,
         )
         self.django_field = django_field
 
@@ -429,43 +473,49 @@ class OrmFileField(OrmConcreteField):
 
 class OrmAggregateField(OrmBaseField):
     def __init__(self, model_name, name, type_):
-        super().__init__(model_name, name, name, type_=type_, concrete=True)
+        super().__init__(
+            model_name, name, name.replace("_", " "), type_=type_, concrete=True
+        )
 
     def bind(self, previous):
         assert previous
-        full_path = previous.full_path + [self.name]
+        queryset_path = previous.queryset_path + [self.name]
         agg_func = _get_django_aggregate(previous.type_, self.name)
         return OrmBoundField(
             field=self,
             previous=previous,
-            full_path=full_path,
+            full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=s(full_path),
-            aggregate_clause=(s(full_path), agg_func(s(previous.full_path))),
+            queryset_path=queryset_path,
+            aggregate_clause=(s(queryset_path), agg_func(s(previous.queryset_path))),
             having=True,
         )
 
 
 class OrmBoundFunctionField(OrmBoundField):
-    def annotate(self, request, qs):
-        func = _get_django_function(self.name)[0](s(self.previous.full_path))
-        return qs.annotate(**{self.queryset_path: func})
+    def _annotate(self, request, qs):
+        func = _get_django_function(self.name)[0](s(self.previous.queryset_path))
+        return qs.annotate(**{s(self.queryset_path): func})
 
 
 class OrmFunctionField(OrmBaseField):
     def __init__(self, model_name, name, type_):
         super().__init__(
-            model_name, name, name, type_=type_, concrete=True, can_pivot=True
+            model_name,
+            name,
+            name.replace("_", " "),
+            type_=type_,
+            concrete=True,
+            can_pivot=True,
         )
 
     def bind(self, previous):
         assert previous
-        full_path = previous.full_path + [self.name]
         return OrmBoundFunctionField(
             field=self,
             previous=previous,
-            full_path=full_path,
+            full_path=previous.full_path + [self.name],
             pretty_path=previous.pretty_path + [self.pretty_name],
-            queryset_path=s(full_path),
+            queryset_path=previous.queryset_path + [self.name],
             filter_=True,
         )
