@@ -11,6 +11,12 @@ ASC, DSC = "asc", "dsc"
 
 
 class TypeMeta(type):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        if self.raw_type and self.element_type:
+            assert self.raw_type.element_type == self.element_type.raw_type
+
     def __repr__(cls):
         return cls.__name__
 
@@ -33,6 +39,9 @@ class TypeMeta(type):
 class BaseType(metaclass=TypeMeta):
     default_value = None
     default_sort = None
+    choices = ()
+    raw_type = None
+    element_type = None
 
     def __init__(self):
         assert False
@@ -42,23 +51,27 @@ class BaseType(metaclass=TypeMeta):
         return {}
 
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: value
 
+    @classmethod
+    def get_formatter(cls, choices):
+        return cls._get_formatter(choices)
+
     @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         return value
 
     @classmethod
-    def parse(cls, lookup, value):
+    def parse(cls, lookup, value, choices):
         lookups = cls.lookups
         if lookup not in lookups:
             return None, f"Bad lookup '{lookup}' expected {lookups}"
         else:
             type_ = TYPES[lookups[lookup]]
             try:
-                return type_._parse(value), None
+                return type_._parse(value, choices), None
             except Exception as e:
                 err_message = str(e) if str(e) else repr(e)
                 return None, err_message
@@ -71,116 +84,49 @@ class BaseType(metaclass=TypeMeta):
 class StringType(BaseType):
     default_value = ""
 
-    @staticmethod
-    def _lookups():
+    @classmethod
+    def _lookups(cls):
         return {
-            "equals": StringType,
-            "contains": StringType,
-            "starts_with": StringType,
-            "ends_with": StringType,
+            "equals": cls,
+            "contains": cls,
+            "starts_with": cls,
+            "ends_with": cls,
             "regex": RegexType,
-            "not_equals": StringType,
-            "not_contains": StringType,
-            "not_starts_with": StringType,
-            "not_ends_with": StringType,
+            "not_equals": cls,
+            "not_contains": cls,
+            "not_starts_with": cls,
+            "not_ends_with": cls,
             "not_regex": RegexType,
-            "is_null": BooleanType,
+            "is_null": IsNullType,
         }
 
 
-class ChoiceTypeMixin:
-    default_value = None
-
-    @staticmethod
-    def get_formatter(choices):
-        assert choices
-        choices = dict(choices)
-        choices[None] = None
-        return lambda value: choices.get(value, value)
-
-
-class StringChoiceType(ChoiceTypeMixin, BaseType):
-    @staticmethod
-    def _lookups():
-        return {
-            **StringType._lookups(),
-            "equals": StringChoiceType,
-            "not_equals": StringChoiceType,
-        }
-
-
-class ArrayTypeMixin:
-    default_value = None
-
-    @staticmethod
-    def get_formatter(choices):  # pragma: postgres
-        if choices:
-            choices = dict(choices)
-            choices[None] = None
-            return (
-                lambda value: None
-                if value is None
-                else ", ".join(str(choices.get(v, v)) for v in value)
-            )
-        return lambda value: None if value is None else ", ".join(str(v) for v in value)
-
-
-class StringArrayType(ArrayTypeMixin, BaseType):
-    @staticmethod
-    def _lookups():
-        return {
-            "contains": StringChoiceType,
-            "length": NumberType,
-            "not_contains": StringChoiceType,
-            "not_length": NumberType,
-            "is_null": BooleanType,
-        }
-
-
-class RegexType(BaseType):
-    default_value = ".*"
-
-    @staticmethod
-    @lru_cache(maxsize=None)
-    def _parse(value):
-        from django.contrib.contenttypes.models import ContentType
-        from django.db.transaction import atomic
-
-        # this is dirty
-        # we need to check if the regex is going to cause a db exception
-        # and not kill any in progress transaction as we check
-        with atomic():
-            list(ContentType.objects.filter(model__regex=value))
-        return value
-
-
 class NumberType(BaseType):
     default_value = 0
 
-    @staticmethod
-    def _lookups():
+    @classmethod
+    def _lookups(cls):
         return {
-            "equals": NumberType,
-            "not_equals": NumberType,
-            "gt": NumberType,
-            "gte": NumberType,
-            "lt": NumberType,
-            "lte": NumberType,
-            "is_null": BooleanType,
+            "equals": cls,
+            "not_equals": cls,
+            "gt": cls,
+            "gte": cls,
+            "lt": cls,
+            "lte": cls,
+            "is_null": IsNullType,
         }
 
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else float(value)
 
     @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         return float(value)
 
     @staticmethod
     def get_format_hints(name, data):
-        # we add . here so there is always at least one .
         nums = [
             row[name] for row in data if row and row[name] and abs(row[name] > 0.0001)
         ]
@@ -192,26 +138,21 @@ class NumberType(BaseType):
         }
 
 
-class NumberChoiceType(ChoiceTypeMixin, BaseType):
-    @staticmethod
-    def _lookups():
-        return {
-            **NumberType._lookups(),
-            "equals": NumberChoiceType,
-            "not_equals": NumberChoiceType,
-        }
+class RegexType(BaseType):
+    default_value = ".*"
 
-
-class NumberArrayType(ArrayTypeMixin, BaseType):
     @staticmethod
-    def _lookups():
-        return {
-            "contains": NumberChoiceType,
-            "length": NumberType,
-            "not_contains": NumberChoiceType,
-            "not_length": NumberType,
-            "is_null": BooleanType,
-        }
+    @lru_cache(maxsize=None)
+    def _parse(value, choices):
+        from django.contrib.contenttypes.models import ContentType
+        from django.db.transaction import atomic
+
+        # this is dirty
+        # we need to check if the regex is going to cause a db exception
+        # and not kill any in progress transaction as we check
+        with atomic():
+            list(ContentType.objects.filter(model__regex=value))
+        return value
 
 
 class YearType(NumberType):
@@ -219,19 +160,7 @@ class YearType(NumberType):
     default_sort = ASC
 
     @staticmethod
-    def _lookups():
-        return {
-            "equals": YearType,
-            "not_equals": YearType,
-            "gt": YearType,
-            "gte": YearType,
-            "lt": YearType,
-            "lte": YearType,
-            "is_null": BooleanType,
-        }
-
-    @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         res = int(value)
         if res <= 1:
             raise Exception("Years must be > 1")
@@ -241,20 +170,20 @@ class YearType(NumberType):
 class DurationType(BaseType):
     default_value = ""
 
-    @staticmethod
-    def _lookups():
+    @classmethod
+    def _lookups(cls):
         return {
-            "equals": DurationType,
-            "not_equals": DurationType,
-            "gt": DurationType,
-            "gte": DurationType,
-            "lt": DurationType,
-            "lte": DurationType,
-            "is_null": BooleanType,
+            "equals": cls,
+            "not_equals": cls,
+            "gt": cls,
+            "gte": cls,
+            "lt": cls,
+            "lte": cls,
+            "is_null": IsNullType,
         }
 
     @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         if value.count(":") == 1:
             value += ":0"
 
@@ -263,7 +192,7 @@ class DurationType(BaseType):
         return res
 
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else str(value)
 
@@ -280,11 +209,11 @@ class DateTypeMixin:
             "gte": cls,
             "lt": cls,
             "lte": cls,
-            "is_null": BooleanType,
+            "is_null": IsNullType,
         }
 
     @classmethod
-    def _parse(cls, value):
+    def _parse(cls, value, choices):
         res = {
             dateutil.parser.parse(value, dayfirst=False, yearfirst=False),
             dateutil.parser.parse(value, dayfirst=True, yearfirst=False),
@@ -298,13 +227,13 @@ class DateTimeType(DateTypeMixin, BaseType):
     default_value = "now"
 
     @classmethod
-    def _parse(cls, value):
+    def _parse(cls, value, choices):
         if value.lower().strip() == "now":
             return timezone.now()
-        return super()._parse(value)
+        return super()._parse(value, choices)
 
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         if settings.USE_TZ:
             return (
@@ -318,13 +247,13 @@ class DateType(DateTypeMixin, BaseType):
     default_value = "today"
 
     @classmethod
-    def _parse(cls, value):
+    def _parse(cls, value, choices):
         if value.lower().strip() == "today":
             return timezone.now().date()
-        return super()._parse(value).date()
+        return super()._parse(value, choices).date()
 
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else str(value)
 
@@ -333,9 +262,9 @@ class WeekDayType(BaseType):
     default_value = "Monday"
     default_sort = ASC
 
-    @staticmethod
-    def _lookups():
-        return {"equals": WeekDayType, "not_equals": WeekDayType}
+    @classmethod
+    def _lookups(cls):
+        return {"equals": cls, "not_equals": cls}
 
     _days = [
         "Sunday",
@@ -348,12 +277,13 @@ class WeekDayType(BaseType):
     ]
 
     @classmethod
-    def get_formatter(cls, value, choices=None):
+    def _get_formatter(cls, choices):
         assert not choices
         return lambda value: None if value is None else cls._days[value - 1]
 
     @classmethod
-    def _parse(cls, value):
+    def _parse(cls, value, choices):
+        assert not choices
         for i, v in enumerate(cls._days):
             if v.lower()[:3] == value.lower()[:3]:
                 return i + 1
@@ -364,9 +294,9 @@ class MonthType(BaseType):
     default_value = "January"
     default_sort = ASC
 
-    @staticmethod
-    def _lookups():
-        return {"equals": MonthType, "not_equals": MonthType}
+    @classmethod
+    def _lookups(cls):
+        return {"equals": cls, "not_equals": cls}
 
     _months = [
         "January",
@@ -384,12 +314,13 @@ class MonthType(BaseType):
     ]
 
     @classmethod
-    def get_formatter(cls, value, choices=None):
+    def _get_formatter(cls, choices):
         assert not choices
         return lambda value: None if value is None else cls._months[value - 1]
 
     @classmethod
-    def _parse(cls, value):
+    def _parse(cls, value, choices):
+        assert not choices
         for i, v in enumerate(cls._months):
             if v.lower()[:3] == value.lower()[:3]:
                 return i + 1
@@ -398,7 +329,11 @@ class MonthType(BaseType):
 
 class HTMLType(StringType):
     @staticmethod
-    def get_formatter(value, choices=None):
+    def _lookups():
+        return {}
+
+    @staticmethod
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else html.conditional_escape(value)
 
@@ -406,16 +341,12 @@ class HTMLType(StringType):
 class BooleanType(BaseType):
     default_value = True
 
-    @staticmethod
-    def _lookups():
-        return {
-            "equals": BooleanType,
-            "not_equals": BooleanType,
-            "is_null": BooleanType,
-        }
+    @classmethod
+    def _lookups(cls):
+        return {"equals": cls, "not_equals": cls, "is_null": IsNullType}
 
     @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         value = value.lower()
         if value == "true":
             return True
@@ -425,65 +356,155 @@ class BooleanType(BaseType):
             raise ValueError("Expected 'true' or 'false'")
 
     @staticmethod
-    def get_formatter(value, choices=None):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else bool(value)
 
 
-class IsNullType(BooleanType):
-    default_value = True
-
-    @staticmethod
-    def _lookups():
-        return {"equals": BooleanType}
-
-    @staticmethod
-    def get_formatter(choices):
-        assert not choices
-        return lambda value: None if value is None else "IsNull" if value else "NotNull"
-
-
 class UnknownType(BaseType):
     @staticmethod
-    def get_formatter(choices):
+    def _get_formatter(choices):
         assert not choices
         return lambda value: None if value is None else str(value)
 
     @staticmethod
     def _lookups():
-        return {"is_null": BooleanType}
+        return {"is_null": IsNullType}
+
+
+def _json_loads(value):
+    try:
+        return json.loads(value.strip())
+    except json.JSONDecodeError:
+        raise ValueError("Invalid JSON value")
 
 
 class JSONFieldType(BaseType):
     default_value = "|"
 
     @staticmethod
-    def _parse(value):
+    def _parse(value, choices):
         value = value.strip()
         if "|" not in value:
             raise ValueError("Missing seperator '|'")
         field, value = value.split("|", 1)
         if not field:
             raise ValueError("Invalid field name")
-        if value.startswith("{") or value.startswith("["):
-            raise ValueError("Not a JSON primitive")
-        try:
-            value = json.loads(value)
-        except json.JSONDecodeError:
-            raise ValueError("Not a JSON primitive")
-        return [field, value]
+        return [field, _json_loads(value)]
 
 
 class JSONType(BaseType):
-    @staticmethod
-    def _lookups():
+    @classmethod
+    def _lookups(cls):
         return {
-            "is_null": BooleanType,
+            "equals": cls,
             "has_key": StringType,
             "field_equals": JSONFieldType,
+            "not_equals": cls,
             "not_has_key": StringType,
             "not_field_equals": JSONFieldType,
+            "is_null": IsNullType,
         }
 
+    @staticmethod
+    def _parse(value, choices):
+        return _json_loads(value)
+
+
+class ChoiceTypeMixin:
+    default_value = None
+
+    @classmethod
+    def _get_formatter(cls, choices):
+        if cls.choices:
+            choices = cls.choices
+
+        assert choices
+        choices = dict(choices)
+        choices[None] = None
+        return lambda value: choices.get(value, value)
+
+    @classmethod
+    def _parse(cls, value, choices):
+        if cls.choices:
+            choices = cls.choices
+
+        assert choices
+        choices = {v: k for k, v in choices}
+        if value not in choices:
+            raise ValueError(f"Unknown choice '{value}'")
+        return choices[value]
+
+    @classmethod
+    def _lookups(cls):
+        return {"equals": cls, "not_equals": cls, "is_null": IsNullType}
+
+
+class StringChoiceType(ChoiceTypeMixin, BaseType):
+    raw_type = StringType
+
+
+class NumberChoiceType(ChoiceTypeMixin, BaseType):
+    raw_type = NumberType
+
+
+class IsNullType(ChoiceTypeMixin, BaseType):
+    choices = [(True, "IsNull"), (False, "NotNull")]
+    default_value = choices[0][1]
+
+    @staticmethod
+    def _lookups():
+        return {"equals": IsNullType}
+
+
+class ArrayTypeMixin:
+    default_value = None
+
+    @classmethod
+    def _get_formatter(cls, choices):  # pragma: postgres
+        element_formatter = cls.element_type._get_formatter(choices)
+        return (
+            lambda value: None
+            if value is None
+            else json.dumps([element_formatter(v) for v in value])
+        )
+
+    @classmethod
+    def _lookups(cls):
+        return {
+            "equals": cls,
+            "contains": cls.element_type,
+            "length": NumberType,
+            "not_equals": cls,
+            "not_contains": cls.element_type,
+            "not_length": NumberType,
+            "is_null": IsNullType,
+        }
+
+    @classmethod
+    def _parse(cls, value, choices):
+        value = _json_loads(value)
+        if not isinstance(value, list):
+            raise ValueError("Expected a list")
+        return [cls.element_type._parse(v, choices) for v in value]
+
+
+class StringArrayType(ArrayTypeMixin, BaseType):
+    element_type = StringType
+
+
+class NumberArrayType(ArrayTypeMixin, BaseType):
+    element_type = NumberType
+
+
+class StringChoiceArrayType(ArrayTypeMixin, BaseType):
+    element_type = StringChoiceType
+    raw_type = StringArrayType
+
+
+class NumberChoiceArrayType(ArrayTypeMixin, BaseType):
+    element_type = NumberChoiceType
+    raw_type = NumberArrayType
+
 
 TYPES = {cls.name: cls for cls in all_subclasses(BaseType)}
