@@ -23,7 +23,7 @@ from .common import HttpResponse, JsonResponse, can_make_public, settings
 from .models import View
 from .orm_admin import OPEN_IN_ADMIN, get_models
 from .orm_results import get_result_queryset, get_results
-from .query import BoundQuery, Query
+from .query import BoundQuery, Query, QueryFilter
 from .types import TYPES
 
 
@@ -31,9 +31,7 @@ def _get_query_data(bound_query):
     return {
         "filters": [
             {
-                "path": filter_.path,
                 "pathStr": filter_.path_str,
-                "prettyPath": filter_.pretty_path,
                 "lookup": filter_.lookup,
                 "value": filter_.value,
             }
@@ -42,9 +40,7 @@ def _get_query_data(bound_query):
         "filterErrors": [filter_.err_message for filter_ in bound_query.filters],
         "fields": [
             {
-                "path": field.path,
                 "pathStr": field.path_str,
-                "prettyPath": field.pretty_path,
                 "sort": field.direction,
                 "priority": field.priority,
                 "pivoted": field.pivoted,
@@ -56,7 +52,9 @@ def _get_query_data(bound_query):
     }
 
 
-def _get_model_fields(orm_model):
+def _get_model_fields(model_name, orm_models):
+    orm_model = orm_models[model_name]
+
     def sort_model_fields(fields):
         front = {"id": 1, OPEN_IN_ADMIN: 2}
         return sorted(fields, key=lambda f: (front.get(f, sys.maxsize), f))
@@ -68,15 +66,25 @@ def _get_model_fields(orm_model):
             "concrete": orm_field.concrete,
             "canPivot": orm_field.can_pivot,
             "prettyName": orm_field.pretty_name,
-            "choices": orm_field.choices,
+            "choices": [v for k, v in orm_field.choices],
         }
         for name, orm_field in orm_model.fields.items()
     }
 
+    q = Query(model_name, [], [QueryFilter(*f) for f in orm_model.default_filters])
+    bq = BoundQuery.bind(q, orm_models)
+
     return {
         "fields": all_fields,
         "sortedFields": sort_model_fields(all_fields),
-        "defaultFilters": orm_model.default_filters,
+        "defaultFilters": [
+            {
+                "pathStr": filter_.path_str,
+                "lookup": filter_.lookup,
+                "value": filter_.value,
+            }
+            for filter_ in bq.filters
+        ],
     }
 
 
@@ -89,13 +97,14 @@ def _get_config(request):
             "defaultLookup": type_.default_lookup,
             "defaultValue": type_.default_value,
             "defaultSort": type_.default_sort,
+            "choices": [v for k, v in type_.choices],
         }
         for name, type_ in TYPES.items()
     }
 
     all_model_fields = {
-        model_name: _get_model_fields(orm_model)
-        for model_name, orm_model in orm_models.items()
+        model_name: _get_model_fields(model_name, orm_models)
+        for model_name in orm_models
     }
 
     return {
