@@ -34,7 +34,7 @@ class NotAnAdminInline(admin.TabularInline):
 
 
 @admin.register(models.Ignored)
-class IgnoredAdmin(admin.ModelAdmin):
+class IgnoredAdmin(AdminMixin, admin.ModelAdmin):
     fields = ["name", "in_admin"]
     ddb_ignore = True
 
@@ -102,7 +102,11 @@ class ProductMixin:
         "tags",
     ]
     readonly_fields = ["id", "is_onsale", "hidden_calculated"]
-    ddb_default_filters = "default=filter&filter=default"
+    ddb_default_filters = [
+        ("a_field", "a_lookup", "a_value"),
+        ("name", "not_equals", "not a thing"),
+        ("a_field", "a_lookup", True),
+    ]
 
     @attributes(ddb_hide=True)
     def hidden_calculated(self, obj):
@@ -160,6 +164,11 @@ class ProductAdmin(AdminMixin, ProductMixin, admin.ModelAdmin):
     def stealth_annotation(self, request, qs):
         return qs.annotate(stealth_annotation=F("name"))
 
+    @annotation
+    @attributes(ddb_hide=True)
+    def hidden_annotation(self, request, qs):
+        return qs.annotate(hidden_annotation=F("name"))
+
     @attributes(boolean=True)
     def calculated_boolean(self, obj):
         return obj.size
