@@ -22,19 +22,28 @@ class JsonAdmin(admin.ModelAdmin):
 
 
 @pytest.fixture
-@pytest.mark.usefixtures("db")
-def get_results_flat(req):
+def with_json(db):
+    admin.site.register(JsonModel, JsonAdmin)
+    yield
+    admin.site.unregister(JsonModel)
+
+
+@pytest.fixture
+def get_results_flat(with_json, req):
     def helper(fields, query=None):
         query = query or {}
 
-        admin.site.register(JsonModel, JsonAdmin)
-        try:
-            orm_models = get_models(req)
-            query = Query.from_request("json.JsonModel", fields, query)
-            bound_query = BoundQuery.bind(query, orm_models)
-            data = get_results(req, bound_query, orm_models, False)
-        finally:
-            admin.site.unregister(JsonModel)
+        orm_models = get_models(req)
+        query = Query.from_request("json.JsonModel", fields, query)
+        bound_query = BoundQuery.bind(query, orm_models)
+        data = get_results(req, bound_query, orm_models, False)
+
+        for f in bound_query.filters:
+            if f.err_message:
+                print(
+                    "filter error:", f.path_str, f.lookup, f.value, "->", f.err_message
+                )
+
         return data["rows"]
 
     return helper
@@ -84,10 +93,11 @@ def test_filter_field_value_no_field(get_results_flat):
 
 
 def test_filter_field_value_list(get_results_flat):
-    JsonModel.objects.create(json_field={"goodbye": "universe"})
+    JsonModel.objects.create(json_field={"goodbye": ["world"]})
+    JsonModel.objects.create(json_field={"goodbye": ["universe"]})
     assert get_results_flat(
         "json_field", {"json_field__field_equals": ['goodbye|["world"]']}
-    ) == [{"json_field": '{"goodbye": "universe"}'}]
+    ) == [{"json_field": '{"goodbye": ["world"]}'}]
 
 
 def test_filter_field_value_bad_json(get_results_flat):
@@ -103,3 +113,19 @@ def test_filter_has_key(get_results_flat):
     assert get_results_flat("json_field", {"json_field__has_key": ["hello"]}) == [
         {"json_field": '{"hello": "world"}'}
     ]
+
+
+def test_filter_equals(get_results_flat):
+    JsonModel.objects.create(json_field={"hello": "world"})
+    JsonModel.objects.create(json_field={"goodbye": "world"})
+    assert get_results_flat(
+        "json_field", {"json_field__equals": ['{"hello": "world"}']}
+    ) == [{"json_field": '{"hello": "world"}'}]
+
+
+def test_filter_equals_bad_json(get_results_flat):
+    JsonModel.objects.create(json_field={"hello": "world"})
+    JsonModel.objects.create(json_field={"goodbye": "world"})
+    assert get_results_flat(
+        "json_field-1", {"json_field__equals": ['{"hello": "world"']}
+    ) == [{"json_field": '{"hello": "world"}'}, {"json_field": '{"goodbye": "world"}'}]
