@@ -92,13 +92,6 @@ def pivot_products(db):
         )
 
 
-@pytest.fixture
-def req(rf, admin_user):
-    req = rf.get("/")
-    req.user = admin_user
-    return req
-
-
 @pytest.fixture
 def orm_models(req):
     return get_models(req)
@@ -222,6 +215,25 @@ def test_get_annotated_field_at_base(products, get_product_flat, mocker):
     assert len(mock.call_args_list) == 2
 
 
+@pytest.mark.skipif(django.VERSION < (2, 1), reason="Django version 2.1 required")
+def test_get_aggretated_annotated_field_at_base(products, get_product_flat, mocker):
+    mock = mocker.patch(
+        "data_browser.orm_results.admin_get_queryset", wraps=admin_get_queryset
+    )
+    data = get_product_flat(1, "annotated__count+1,size-2", {})
+    assert data == [[1, 2], [2, 1]]
+    assert len(mock.call_args_list) == 2
+
+
+def test_get_annotated_field_function_at_base(products, get_product_flat, mocker):
+    mock = mocker.patch(
+        "data_browser.orm_results.admin_get_queryset", wraps=admin_get_queryset
+    )
+    data = get_product_flat(1, "annotated__is_null+1,size-2", {})
+    assert data == [["NotNull", 2.0], ["NotNull", 1.0]]
+    assert len(mock.call_args_list) == 2
+
+
 def test_get_annotated_field_down_tree(products, get_product_flat, mocker):
     mock = mocker.patch(
         "data_browser.orm_results.admin_get_queryset", wraps=admin_get_queryset
@@ -235,6 +247,25 @@ def test_get_annotated_field_down_tree(products, get_product_flat, mocker):
     assert len(mock.call_args_list) == 2
 
 
+@pytest.mark.skipif(django.VERSION < (2, 1), reason="Django version 2.1 required")
+def test_get_aggregated_annotated_field_down_tree(products, get_product_flat, mocker):
+    mock = mocker.patch(
+        "data_browser.orm_results.admin_get_queryset", wraps=admin_get_queryset
+    )
+    data = get_product_flat(1, "producer__address__andrew__count+1,size-2", {})
+    assert data == [[0, 2], [2, 1]]
+    assert len(mock.call_args_list) == 2
+
+
+def test_get_annotated_field_function_down_tree(products, get_product_flat, mocker):
+    mock = mocker.patch(
+        "data_browser.orm_results.admin_get_queryset", wraps=admin_get_queryset
+    )
+    data = get_product_flat(1, "producer__address__andrew__is_null+1,size-2", {})
+    assert data == [["NotNull", 1.0], ["IsNull", 2.0]]
+    assert len(mock.call_args_list) == 2
+
+
 @pytest.mark.usefixtures("products")
 def test_get_multiple_calculated_fields_on_admins(get_product_flat):
     data = get_product_flat(3, "producer__address__bob,producer__frank", {})
@@ -398,13 +429,9 @@ def test_get_results_collapsed(get_product_flat):
 
 @pytest.mark.usefixtures("products")
 def test_get_results_null_filter(get_product_flat):
-    data = get_product_flat(1, "name", {"onsale__is_null": ["True"]})
+    data = get_product_flat(1, "name", {"onsale__is_null": ["IsNull"]})
     sortedAssert(data, [["a"], ["b"], ["c"]])
-    data = get_product_flat(1, "name", {"onsale__is_null": ["true"]})
-    sortedAssert(data, [["a"], ["b"], ["c"]])
-    data = get_product_flat(1, "name", {"onsale__is_null": ["False"]})
-    sortedAssert(data, [])
-    data = get_product_flat(1, "name", {"onsale__is_null": ["false"]})
+    data = get_product_flat(1, "name", {"onsale__is_null": ["NotNull"]})
     sortedAssert(data, [])
 
 
@@ -818,3 +845,31 @@ def test_duration_aggregations(get_product_flat, aggregation, value):
 
     data = get_product_flat(1, f"duration__{aggregation}", {})
     assert data == [[value]]
+
+
+def test_string_choice_field(get_product_flat):
+    producer = models.Producer.objects.create()
+    models.Product.objects.create(producer=producer, string_choice="a")
+    models.Product.objects.create(producer=producer, string_choice="b")
+
+    data = get_product_flat(1, f"string_choice", {"string_choice__equals": ["A"]})
+    assert data == [["A"]]
+
+    data = get_product_flat(
+        1, f"string_choice__raw", {"string_choice__raw__equals": ["a"]}
+    )
+    assert data == [["a"]]
+
+
+def test_number_choice_field(get_product_flat):
+    producer = models.Producer.objects.create()
+    models.Product.objects.create(producer=producer, number_choice=1)
+    models.Product.objects.create(producer=producer, number_choice=2)
+
+    data = get_product_flat(1, f"number_choice", {"number_choice__equals": ["A"]})
+    assert data == [["A"]]
+
+    data = get_product_flat(
+        1, f"number_choice__raw", {"number_choice__raw__equals": [1]}
+    )
+    assert data == [[1]]
